searchData={"items":[{"type":"module","title":"Peri","doc":"Peri is a schema validation library for Elixir, inspired by Clojure's Plumatic Schema.\nIt provides a flexible and powerful way to define and validate data structures using schemas. \nThe library supports nested schemas, optional fields, custom validation functions, and various type constraints.","ref":"Peri.html"},{"type":"module","title":"Key Features - Peri","doc":"- **Simple and Nested Schemas**: Define schemas that can handle complex, nested data structures.\n- **Optional and Required Fields**: Specify fields as optional or required with type constraints.\n- **Custom Validation Functions**: Use custom functions to validate fields.\n- **Comprehensive Error Handling**: Provides detailed error messages for validation failures.\n- **Type Constraints**: Supports various types including enums, lists, tuples, and more.","ref":"Peri.html#module-key-features"},{"type":"module","title":"Usage - Peri","doc":"To define a schema, use the `defschema` macro. By default, all fields in the schema are optional unless specified otherwise.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user, %{\n    name: :string,\n    age: :integer,\n    email: {:required, :string},\n    address: %{\n      street: :string,\n      city: :string\n    },\n    tags: {:list, :string},\n    role: {:enum, [:admin, :user, :guest]},\n    geolocation: {:tuple, [:float, :float]},\n    rating: {:custom, &validate_rating/1}\n  }\n\n  defp validate_rating(n) when n < 10, do: :ok\n  defp validate_rating(_), do: {:error, \"invalid rating\", []}\nend\n```\n\nYou can then use the schema to validate data:\n\n```elixir\nuser_data = %{\n  name: \"John\", age: 30, email: \"john@example.com\",\n  address: %{street: \"123 Main St\", city: \"Somewhere\"},\n  tags: [\"science\", \"funky\"], role: :admin,\n  geolocation: {12.2, 34.2}, rating: 9\n}\n\ncase MySchemas.user(user_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","ref":"Peri.html#module-usage"},{"type":"module","title":"Available Types - Peri","doc":"- `:string` - Validates that the field is a binary (string).\n- `:integer` - Validates that the field is an integer.\n- `:float` - Validates that the field is a float.\n- `:boolean` - Validates that the field is a boolean.\n- `:atom` - Validates that the field is an atom.\n- `:any` - Allows any datatype.\n- `{:required, type}` - Marks the field as required and validates it according to the specified type.\n- `:map` - Validates that the field is a map.\n- `{:either, {type_1, type_2}}` - Validates that the field is either of `type_1` or `type_2`.\n- `{:oneof, types}` - Validates that the field is at least one of the provided types.\n- `{:list, type}` - Validates that the field is a list with elements of the specified type.\n- `{:tuple, types}` - Validates that the field is a tuple with the specified types in sequence.\n- `{:custom, anonymous_fun_arity_1}` - Validates that the field passes the callback function.\n- `{:custom, {MyModule, :my_validation}}` - Validates using a function from a specific module.\n- `{:custom, {MyModule, :my_validation, [arg1, arg2]}}` - Same as above but allows extra arguments.\n-\t`{:cond, condition, true_type, else_type}` - Validates the field based on a condition.\n-\t`{:dependent, field, condition, type}` - Validates the field based on another field’s value.\n-\t`{type, {:default, default}}` - Sets a default value for a field if it is nil.","ref":"Peri.html#module-available-types"},{"type":"module","title":"Error Handling - Peri","doc":"Peri provides detailed error messages that include the path to the invalid data, the expected and actual values, and custom error messages for custom validations.","ref":"Peri.html#module-error-handling"},{"type":"module","title":"Functions - Peri","doc":"- `validate/2` - Validates data against a schema.\n- `conforms?/2` - Checks if data conforms to a schema.\n- `validate_schema/1` - Validates the schema definition.","ref":"Peri.html#module-functions"},{"type":"module","title":"Example - Peri","doc":"```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user, %{\n    name: :string,\n    age: :integer,\n    email: {:required, :string}\n  }\nend\n\nuser_data = %{name: \"John\", age: 30, email: \"john@example.com\"}\ncase MySchemas.user(user_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","ref":"Peri.html#module-example"},{"type":"function","title":"Peri.conforms?/2","doc":"","ref":"Peri.html#conforms?/2"},{"type":"macro","title":"Peri.defschema/2","doc":"Defines a schema with a given name and schema definition.","ref":"Peri.html#defschema/2"},{"type":"macro","title":"Examples - Peri.defschema/2","doc":"defmodule MySchemas do\n      import Peri\n\n      defschema :user, %{\n        name: :string,\n        age: :integer,\n        email: {:required, :string}\n      }\n    end\n\n    user_data = %{name: \"John\", age: 30, email: \"john@example.com\"}\n    MySchemas.user(user_data)\n    # => {:ok, %{name: \"John\", age: 30, email: \"john@example.com\"}}\n\n    invalid_data = %{name: \"John\", age: 30}\n    MySchemas.user(invalid_data)\n    # => {:error, [email: \"is required\"]}","ref":"Peri.html#defschema/2-examples"},{"type":"function","title":"Peri.validate/2","doc":"Validates a given data map against a schema.\n\nReturns `{:ok, data}` if the data is valid according to the schema, or `{:error, errors}` if there are validation errors.","ref":"Peri.html#validate/2"},{"type":"function","title":"Parameters - Peri.validate/2","doc":"- schema: The schema definition map.\n  - data: The data map to be validated.","ref":"Peri.html#validate/2-parameters"},{"type":"function","title":"Examples - Peri.validate/2","doc":"schema = %{\n      name: :string,\n      age: :integer,\n      email: {:required, :string}\n    }\n\n    data = %{name: \"John\", age: 30, email: \"john@example.com\"}\n    Peri.validate(schema, data)\n    # => {:ok, %{name: \"John\", age: 30, email: \"john@example.com\"}}\n\n    invalid_data = %{name: \"John\", age: 30}\n    Peri.validate(schema, invalid_data)\n    # => {:error, [email: \"is required\"]}","ref":"Peri.html#validate/2-examples"},{"type":"function","title":"Peri.validate_schema/1","doc":"Validates a schema definition to ensure it adheres to the expected structure and types.\n\nThis function can handle both simple and complex schema definitions, including nested schemas, custom validation functions, and various type constraints.","ref":"Peri.html#validate_schema/1"},{"type":"function","title":"Parameters - Peri.validate_schema/1","doc":"- `schema` - The schema definition to be validated. It can be a map or a keyword list representing the schema.","ref":"Peri.html#validate_schema/1-parameters"},{"type":"function","title":"Returns - Peri.validate_schema/1","doc":"- `{:ok, schema}` - If the schema is valid, returns the original schema.\n  - `{:error, errors}` - If the schema is invalid, returns an error tuple with detailed error information.","ref":"Peri.html#validate_schema/1-returns"},{"type":"function","title":"Examples - Peri.validate_schema/1","doc":"Validating a simple schema:\n  \n  ```elixir\n  schema = %{\n    name: :string,\n    age: :integer,\n    email: {:required, :string}\n  }\n  assert {:ok, ^schema} = validate_schema(schema)\n  ```\n\n  Validating a nested schema:\n\n  ```elixir\n  schema = %{\n    user: %{\n      name: :string,\n      profile: %{\n        age: {:required, :integer},\n        email: {:required, :string}\n      }\n    }\n  }\n  assert {:ok, ^schema} = validate_schema(schema)\n  ```\n\n  Handling invalid schema definition:\n\n  ```elixir\n  schema = %{\n    name: :str,\n    age: :integer,\n    email: {:required, :string}\n  }\n  assert {:error, _errors} = validate_schema(schema)\n  ```","ref":"Peri.html#validate_schema/1-examples"},{"type":"module","title":"Peri.Error","doc":"Defines the structure and functions for handling validation errors in the Peri schema validation library.\n\nThe `Peri.Error` module encapsulates information about validation errors that occur during schema validation. Each error contains details about the path to the invalid data, the type of error, and any nested errors for complex or deeply nested schemas.","ref":"Peri.Error.html"},{"type":"module","title":"Attributes - Peri.Error","doc":"- `:message` - A human-readable message describing the error.\n- `:content` - Additional information about the error, such as expected and actual values.\n- `:path` - A list representing the path to the invalid data within the structure being validated.\n- `:key` - The specific key or field that caused the error.\n- `:errors` - A list of nested `Peri.Error` structs for detailed information about nested validation errors.","ref":"Peri.Error.html#module-attributes"},{"type":"module","title":"Example - Peri.Error","doc":"iex> error = %Peri.Error{\n    ...>   message: \"Validation failed\",\n    ...>   content: %{expected: :string, actual: :integer},\n    ...>   path: [:user, :age],\n    ...>   key: :age,\n    ...>   errors: [\n    ...>     %Peri.Error{\n    ...>       message: \"Expected type string, got integer\",\n    ...>       content: nil,\n    ...>       path: [:user, :age],\n    ...>       key: :age,\n    ...>       errors: nil\n    ...>     }\n    ...>   ]\n    ...> }\n    %Peri.Error{\n      message: \"Validation failed\",\n      content: %{expected: :string, actual: :integer},\n      path: [:user, :age],\n      key: :age,\n      errors: [\n        %Peri.Error{\n          message: \"Expected type string, got integer\",\n          content: nil,\n          path: [:user, :age],\n          key: :age,\n          errors: nil\n        }\n      ]\n    }","ref":"Peri.Error.html#module-example"},{"type":"module","title":"Functions - Peri.Error","doc":"- `error_to_map/1` - Converts a `Peri.Error` struct to a map, including nested errors.","ref":"Peri.Error.html#module-functions"},{"type":"function","title":"Peri.Error.error_to_map/1","doc":"Recursively converts a `Peri.Error` struct into a map.","ref":"Peri.Error.html#error_to_map/1"},{"type":"function","title":"Parameters - Peri.Error.error_to_map/1","doc":"- `error` - A `Peri.Error` struct to be transformed.","ref":"Peri.Error.html#error_to_map/1-parameters"},{"type":"function","title":"Examples - Peri.Error.error_to_map/1","doc":"iex> error = %Peri.Error{\n    ...>   message: \"Validation failed\",\n    ...>   content: %{expected: :string, actual: :integer},\n    ...>   path: [:user, :age],\n    ...>   key: :age,\n    ...>   errors: [\n    ...>     %Peri.Error{\n    ...>       message: \"Expected type string, got integer\",\n    ...>       content: nil,\n    ...>       path: [:user, :age],\n    ...>       key: :age,\n    ...>       errors: nil\n    ...>     }\n    ...>   ]\n    ...> }\n    iex> Peri.Error.error_to_map(error)\n    %{\n      message: \"Validation failed\",\n      content: %{expected: :string, actual: :integer},\n      path: [:user, :age],\n      key: :age,\n      errors: [\n        %{\n          message: \"Expected type string, got integer\",\n          content: nil,\n          path: [:user, :age],\n          key: :age,\n          errors: nil\n        }\n      ]\n    }","ref":"Peri.Error.html#error_to_map/1-examples"},{"type":"function","title":"Peri.Error.format_error_message/2","doc":"","ref":"Peri.Error.html#format_error_message/2"},{"type":"function","title":"Peri.Error.new_child/4","doc":"Creates a new child error with a path, key, message, and context.","ref":"Peri.Error.html#new_child/4"},{"type":"function","title":"Parameters - Peri.Error.new_child/4","doc":"- `path` - A list representing the path to the invalid data.\n  - `key` - The specific key or field that caused the error.\n  - `message` - A string template for the error message.\n  - `context` - A list of key-value pairs to replace in the message template.","ref":"Peri.Error.html#new_child/4-parameters"},{"type":"function","title":"Examples - Peri.Error.new_child/4","doc":"iex> Peri.Error.new_child([:user], :age, \"Invalid value for %{field}\", [field: \"age\"])\n    %Peri.Error{\n      path: [:user, :age],\n      key: :age,\n      message: \"Invalid value for age\",\n      content: %{field: \"age\"}\n    }","ref":"Peri.Error.html#new_child/4-examples"},{"type":"function","title":"Peri.Error.new_parent/3","doc":"Creates a new parent error with nested errors.","ref":"Peri.Error.html#new_parent/3"},{"type":"function","title":"Parameters - Peri.Error.new_parent/3","doc":"- `path` - A list representing the path to the invalid data.\n  - `key` - The specific key or field that caused the error.\n  - `errors` - A list of nested `Peri.Error` structs.","ref":"Peri.Error.html#new_parent/3-parameters"},{"type":"function","title":"Examples - Peri.Error.new_parent/3","doc":"iex> Peri.Error.new_parent([:user], :age, [%Peri.Error{message: \"Invalid age\"}])\n    %Peri.Error{\n      path: [:user, :age],\n      key: :age,\n      errors: [%Peri.Error{message: \"Invalid age\"}]\n    }","ref":"Peri.Error.html#new_parent/3-examples"},{"type":"function","title":"Peri.Error.new_single/2","doc":"Creates a new single error with a formatted message and context.","ref":"Peri.Error.html#new_single/2"},{"type":"function","title":"Parameters - Peri.Error.new_single/2","doc":"- `message` - A string template for the error message.\n  - `context` - A list of key-value pairs to replace in the message template.","ref":"Peri.Error.html#new_single/2-parameters"},{"type":"function","title":"Examples - Peri.Error.new_single/2","doc":"iex> Peri.Error.new_single(\"Invalid value for %{field}\", [field: \"age\"])\n    %Peri.Error{\n      message: \"Invalid value for age\",\n      content: %{field: \"age\"}\n    }","ref":"Peri.Error.html#new_single/2-examples"},{"type":"function","title":"Peri.Error.update_error_paths/2","doc":"","ref":"Peri.Error.html#update_error_paths/2"},{"type":"type","title":"Peri.Error.t/0","doc":"","ref":"Peri.Error.html#t:t/0"},{"type":"module","title":"Peri.Parser","doc":"The `Peri.Parser` module is responsible for managing the state during schema validation. \nIt centralizes functions to handle updating data, adding errors, and managing the path within \nthe data structure being validated.","ref":"Peri.Parser.html"},{"type":"module","title":"Struct - Peri.Parser","doc":"The `Peri.Parser` struct has the following fields:\n\n- `:data` - The current state of the data being validated.\n- `:errors` - A list of errors encountered during validation.\n- `:path` - The current path within the data structure being validated.","ref":"Peri.Parser.html#module-struct"},{"type":"function","title":"Peri.Parser.add_error/2","doc":"Adds an error to the parser state's list of errors.","ref":"Peri.Parser.html#add_error/2"},{"type":"function","title":"Parameters - Peri.Parser.add_error/2","doc":"- `state` - The current `Peri.Parser` state.\n- `err` - The `%Peri.Error{}` struct representing the error to add.","ref":"Peri.Parser.html#add_error/2-parameters"},{"type":"function","title":"Examples - Peri.Parser.add_error/2","doc":"iex> state = Peri.Parser.new(%{name: \"Alice\"})\n    iex> error = %Peri.Error{path: [:name], message: \"is required\", content: []}\n    iex> Peri.Parser.add_error(state, error)\n    %Peri.Parser{data: %{name: \"Alice\"}, errors: [%Peri.Error{path: [:name], message: \"is required\", content: []}], path: []}","ref":"Peri.Parser.html#add_error/2-examples"},{"type":"function","title":"Peri.Parser.new/1","doc":"Initializes a new `Peri.Parser` struct with the given data.","ref":"Peri.Parser.html#new/1"},{"type":"function","title":"Parameters - Peri.Parser.new/1","doc":"- `data` - The initial data to be validated.","ref":"Peri.Parser.html#new/1-parameters"},{"type":"function","title":"Examples - Peri.Parser.new/1","doc":"iex> Peri.Parser.new(%{name: \"Alice\"})\n    %Peri.Parser{data: %{name: \"Alice\"}, errors: [], path: []}","ref":"Peri.Parser.html#new/1-examples"},{"type":"function","title":"Peri.Parser.update_data/3","doc":"Updates the data in the parser state at the given key with the specified value.","ref":"Peri.Parser.html#update_data/3"},{"type":"function","title":"Parameters - Peri.Parser.update_data/3","doc":"- `state` - The current `Peri.Parser` state.\n- `key` - The key to update in the data.\n- `val` - The value to set at the specified key.","ref":"Peri.Parser.html#update_data/3-parameters"},{"type":"function","title":"Examples - Peri.Parser.update_data/3","doc":"iex> state = Peri.Parser.new(%{name: \"Alice\"})\n    iex> Peri.Parser.update_data(state, :age, 30)\n    %Peri.Parser{data: %{name: \"Alice\", age: 30}, errors: [], path: []}","ref":"Peri.Parser.html#update_data/3-examples"},{"type":"exception","title":"Peri.InvalidSchema","doc":"Exception raised when an invalid schema is encountered.\n\nThis exception is raised with a list of `Peri.Error` structs,\nproviding a readable message overview of the validation errors.","ref":"Peri.InvalidSchema.html"},{"type":"type","title":"Peri.InvalidSchema.t/0","doc":"","ref":"Peri.InvalidSchema.html#t:t/0"},{"type":"extras","title":"Peri - Schema Validation Library for Elixir","doc":"# Peri - Schema Validation Library for Elixir","ref":"readme.html"},{"type":"extras","title":"General Description - Peri - Schema Validation Library for Elixir","doc":"Peri is a schema validation library for Elixir, inspired by Clojure's Plumatic Schema. It allows developers to define schemas for validating various data structures, supporting nested schemas, optional fields, and custom validation types. Peri aims to provide an intuitive and flexible way to ensure data integrity in Elixir applications.","ref":"readme.html#general-description"},{"type":"extras","title":"Features - Peri - Schema Validation Library for Elixir","doc":"- Simple and intuitive syntax for defining schemas.\n- Validation of data structures against schemas.\n- Support for nested, composable, and recursive schemas.\n- Optional and required fields.\n- Comprehensive error handling with detailed messages.\n- Flexible validation types, including custom and conditional validations.","ref":"readme.html#features"},{"type":"extras","title":"Installation - Peri - Schema Validation Library for Elixir","doc":"To use Peri in your project, add it to your dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:peri, \"~> 0.2.3\"}\n  ]\nend\n```\n\nThen, run `mix deps.get` to fetch the dependencies.","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Peri - Schema Validation Library for Elixir","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Available Types - Peri - Schema Validation Library for Elixir","doc":"Peri supports a variety of types to ensure your data is validated accurately. Below is a table summarizing the available types and their descriptions:\n\n| Type                                      | Description                                                                                     |\n|-------------------------------------------|-------------------------------------------------------------------------------------------------|\n| `:string`                                 | Validates that the field is a binary (string).                                                  |\n| `:integer`                                | Validates that the field is an integer.                                                         |\n| `:float`                                  | Validates that the field is a float.                                                            |\n| `:boolean`                                | Validates that the field is a boolean.                                                          |\n| `:atom`                                   | Validates that the field is an atom.                                                            |\n| `:any`                                    | Allows any datatype.                                                                            |\n| `{:required, type}`                       | Marks the field as required and validates it according to the specified type.                   |\n| `:map`                                    | Validates that the field is a map without checking nested schema.                               |\n| `{:either, {type_1, type_2}}`             | Validates that the field is either of `type_1` or `type_2`.                                     |\n| `{:oneof, types}`                         | Validates that the field is at least one of the provided types.                                 |\n| `{:list, type}`                           | Validates that the field is a list where elements belong to a determined type.                  |\n| `{:tuple, types}`                         | Validates that the field is a tuple with a determined size, and each element has its own type validation. |\n| `{:custom, anonymous_fun_arity_1}`        | Validates that the field passes the callback. The function needs to return either `:ok` or `{:error, template, info}` where `template` is an EEx string and `info` is a keyword list or map. |\n| `{:custom, {MyModule, :my_validation}}`   | Same as `{custom, anonymous_fun_arity_1}` but you pass a remote module and a function name as an atom. |\n| `{:custom, {MyModule, :my_validation, [arg1, arg2]}}` | Same as `{:custom, {MyModule, :my_validation}}` but you can pass extra arguments to your validation function. Note that the value of the field is always the first argument. |\n| `{:cond, condition, true_type, else_type}` | Conditionally validates a field based on the result of a condition function.                    |\n| `{:dependent, field, condition, type}`    | Validates a field based on the value of another field.                                          |\n| `{type, {:default, default}}`             | Validates a field exists based on `type`, if not, return the `default` value                                          |\n| `{type, {:transform, mapper}}`             | Validates a field have valid `type`, if yes, return the return of the `mapper/1` function passing the value                                         |\n\nThese types provide flexibility and control over how data is validated, enabling robust and precise schema definitions.","ref":"readme.html#available-types"},{"type":"extras","title":"Defining and Validating Schemas - Peri - Schema Validation Library for Elixir","doc":"Schemas can be defined using the `defschema` macro. By default, all fields in the schema are optional unless specified as `{:required, type}`.\n\n#### Example\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user, %{\n    name: :string,\n    age: :integer,\n    email: {:required, :string},\n    address: %{\n      street: :string,\n      city: :string\n    },\n    tags: {:list, :string},\n    role: {:enum, [:admin, :user, :guest]},\n    geolocation: {:tuple, [:float, :float]},\n    rating: {:custom, &validate_rating/1}\n  }\n\n  defp validate_rating(n) when n < 10, do: :ok\n  defp validate_rating(_), do: {:error, \"invalid rating\", []}\nend\n\nuser_data = %{\n  name: \"John\",\n  age: 30,\n  email: \"john@example.com\",\n  address: %{street: \"123 Main St\", city: \"Somewhere\"},\n  tags: [\"science\", \"funky\"],\n  role: :admin,\n  geolocation: {12.2, 34.2},\n  rating: 9\n}\n\ncase MySchemas.user(user_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```\n\n### `defschema` Macro General Explanation\n\nThe `defschema` macro allows you to define a schema with a given name and schema definition. This macro injects functions that can validate data against the defined schema.","ref":"readme.html#defining-and-validating-schemas"},{"type":"extras","title":"Defining Schemas Without Macro - Peri - Schema Validation Library for Elixir","doc":"You can also define schemas without using the `defschema` macro by directly passing the schema definition to the `Peri.validate/2` function.\n\n```elixir\ndefmodule MySchemas do\n  @raw_user_schema %{age: :integer, name: :string}\n\n  def create_user(data) do\n    with {:ok, data} <- Peri.validate(@raw_user_schema, data) do\n      # rest of the function ...\n    end\n  end\nend\n```","ref":"readme.html#defining-schemas-without-macro"},{"type":"extras","title":"Dynamic Schemas - Peri - Schema Validation Library for Elixir","doc":"Dynamic schemas can be generated based on runtime conditions.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  def generate_schema(is_admin) do\n    if is_admin do\n      %{\n        role: {:required, :string},\n        permissions: {:list, :string}\n      }\n    else\n      %{\n        role: {:required, :string}\n      }\n    end\n  end\n\n  def validate_user(data, is_admin) do\n    schema = generate_schema(is_admin)\n    Peri.validate(schema, data)\n  end\nend\n\ndata = %{role: \"admin\", permissions: [\"read\", \"write\"]}\ncase MySchemas.validate_user(data, true) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","ref":"readme.html#dynamic-schemas"},{"type":"extras","title":"Nested and Composable Schemas - Peri - Schema Validation Library for Elixir","doc":"Peri supports nested schemas, allowing for validation of complex data structures.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :address, %{\n    street: :string,\n    city: :string\n  }\n\n  defschema :user, %{\n    name: :string,\n    email: {:required, :string},\n    address: {:custom, &address/1}\n  }\nend\n\ndata = %{name: \"John\", email: \"john@example.com\", address: %{street: \"123 Main St\", city: \"Somewhere\"}}\ncase MySchemas.user(data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","ref":"readme.html#nested-and-composable-schemas"},{"type":"extras","title":"Recursive Schemas - Peri - Schema Validation Library for Elixir","doc":"Recursive schemas allow you to define schemas that reference themselves, enabling the validation of nested and hierarchical data structures.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :category, %{\n    id: :integer,\n    name: :string,\n    subcategories: {:list, {:custom, &category/1}}\n  }\n\n  category_data = %{\n    id: 1,\n    name: \"Electronics\",\n    subcategories: [\n      %{id: 2, name: \"Computers\", subcategories: []},\n      %{id: 3, name: \"Phones\", subcategories: [%{id: 4, name: \"Smartphones\", subcategories: []}]}\n    ]\n  }\n\n  case MySchemas.category(category_data) do\n    {:ok, valid_data} -> IO.puts(\"Category is valid!\")\n    {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\n  end\nend\n```","ref":"readme.html#recursive-schemas"},{"type":"extras","title":"Schemas on Raw Data Structures - Peri - Schema Validation Library for Elixir","doc":"Peri allows you to define schemas for various data structures, including lists, tuples, keyword lists, and primitive types.\n\n#### Lists\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :string_list, {:list, :string}\n\n  data = [\"hello\", \"world\"]\n  case MySchemas.string_list(data) do\n    {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n    {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\n  end\nend\n```\n\n#### Tuples\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :coordinates, {:tuple, [:float, :float]}\n\n  data = {12.34, 56.78}\n  case MySchemas.coordinates(data) do\n    {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n    {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\n  end\nend\n```\n\n#### Keyword Lists\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :settings, [{:key, :string}, {:value, :any}]\n\n  data = [key: \"theme\", value: \"dark\"]\n  case MySchemas.settings(data) do\n    {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n    {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\n  end\nend\n```","ref":"readme.html#schemas-on-raw-data-structures"},{"type":"extras","title":"Error Handling - Peri - Schema Validation Library for Elixir","doc":"Peri provides detailed error messages that can be easily inspected and transformed. Each error includes a message, content, path, key, and nested errors for detailed information about nested validation errors.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user, %{\n    name: :string,\n    age: {:required, :integer}\n  }\n\n  data = %{name: \"Jane\"}\n  case MySchemas.user(data) do\n    {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n    {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\n  end\nend\n```","ref":"readme.html#error-handling"},{"type":"extras","title":"InvalidSchema Exception - Peri - Schema Validation Library for Elixir","doc":"Peri raises an `InvalidSchema` exception when an invalid schema is encountered. This exception contains a list of `Peri.Error` structs, providing a readable message overview of the validation errors.","ref":"readme.html#invalidschema-exception"},{"type":"extras","title":"Comparison with Ecto Schemaless Changesets and Embedded Schemas - Peri - Schema Validation Library for Elixir","doc":"","ref":"readme.html#comparison-with-ecto-schemaless-changesets-and-embedded-schemas"},{"type":"extras","title":"Peri - Peri - Schema Validation Library for Elixir","doc":"- **Purpose**: Designed specifically for schema validation. Focuses on validating raw maps against defined schemas.\n- **Flexibility**: Allows easy validation of nested structures, optional fields, and dynamic schemas.\n- **Simplicity**: The syntax for defining schemas is straightforward and intuitive.\n- **Use Case**: Ideal for validating data structures in contexts where you don't need the full power of a database ORM.","ref":"readme.html#peri"},{"type":"extras","title":"Ecto Schemaless Changesets - Peri - Schema Validation Library for Elixir","doc":"- **Purpose**: Provides mechanisms for validating and casting data without persisting it to a database.\n- **Complexity**: More complex due to its integration with Ecto and the need to handle changesets.\n- **Schema Definitions**: Uses Ecto changesets and embedded schemas, which are typically tied to database schemas.\n- **Use Case**: Ideal for applications that require validation and casting of data, even when it’s not being persisted to a database.","ref":"readme.html#ecto-schemaless-changesets"},{"type":"extras","title":"Summary - Peri - Schema Validation Library for Elixir","doc":"- Use **Peri** if you need a lightweight, flexible library for validating raw maps and nested data structures without the overhead of database interactions.\n- Use **Ecto Schemaless Changesets** if you need to validate and cast data in an Ecto-based application, leveraging the full power of Ecto’s changeset functionality.","ref":"readme.html#summary"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}